from collections import namedtuple

import numpy as np
import torch
from sentence_transformers import (
    CrossEncoder,
    InputExample,
    SentenceTransformer,
    datasets,
    losses,
    models,
    util,
)
import faiss
import urllib
import zipfile
import os
import pandas as pd

model = namedtuple("model", ["url", "model"])
models = {
    "retrieve_rank": model(
        url="https://github.com/Nandhagopalan/Semanticsearch/releases/download/0.0.2/retrieve_rerank.zip",
        model=SentenceTransformer,
    ),
    "faiss_index":model(
        url="https://github.com/Nandhagopalan/Semanticsearch/releases/download/0.0.2/search.index.zip",
        model=faiss
    ),
     "data":model(
        url="https://github.com/Nandhagopalan/Semanticsearch/releases/download/0.0.2/covid_papers.csv",
        model=pd
    )
}

def list_models():
    """
    Print all available pretrained models
    """
    return list(models.keys())


def get_model(embedder,faissix,data):
    """
    Load the pretrained weights and return search results
    Example:
    query = ''
    model: RetrieveRerank = get_model("retrieve_rank")

    model.query_as_json(query)
    """
    if not models.get(embedder):
        raise Exception("Model name not found!")

    model_class = models[embedder].model(models[embedder].url)
    
    ### check if file exist and dont download again
    if not os.path.isfile('../model/search.index'):
    
        urllib.request.urlretrieve(models[faissix].url, "search.index.zip")

        with zipfile.ZipFile("search.index.zip", 'r') as zip_ref:
            zip_ref.extractall('../model/')

        os.remove("search.index.zip")

    if not os.path.isfile('covid_papers.csv'):
        urllib.request.urlretrieve(models[data].url, "covid_papers.csv")

    data=pd.read_csv('covid_papers.csv')
    passages=data['abstract'].values.tolist()

    os.remove("covid_papers.csv")

    index=models[faissix].model.read_index('../model/search.index')
    
    return model_class,index,passages



import math
import os
import random
import time

import numpy as np
import torch
from sklearn import metrics

OUTPUT_DIR = "./"


def get_score(y_true, y_pred):
    return metrics.accuracy_score(y_true, y_pred)


def init_logger(log_file=OUTPUT_DIR + "train.log"):
    from logging import INFO, FileHandler, Formatter, StreamHandler, getLogger

    logger = getLogger(__name__)
    logger.setLevel(INFO)
    handler1 = StreamHandler()
    handler1.setFormatter(Formatter("%(message)s"))
    handler2 = FileHandler(filename=log_file)
    handler2.setFormatter(Formatter("%(message)s"))
    logger.addHandler(handler1)
    logger.addHandler(handler2)
    return logger


def seed_torch(seed=42):
    random.seed(seed)
    os.environ["PYTHONHASHSEED"] = str(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    torch.cuda.manual_seed(seed)
    torch.backends.cudnn.deterministic = True


def time_since(since, percent):
    now = time.time()
    s = now - since
    es = s / (percent)
    rs = es - s
    return "{} (remain {})".format(as_minutes(s), as_minutes(rs))


def as_minutes(s):
    m = math.floor(s / 60)
    s -= m * 60
    return "%dm %ds" % (m, s)


class AverageMeter:
    """Computes and stores the average and current value"""

    def __init__(self):
        self.reset()

    def reset(self):
        self.val = 0
        self.avg = 0
        self.sum = 0
        self.count = 0

    def update(self, val, n=1):
        self.val = val
        self.sum += val * n
        self.count += n
        self.avg = self.sum / self.count
